; ================================================================================
; 						Définitions des fonctions
; 					spécifiques pour FramaWizahk
; ================================================================================


;===============================================================================
; Function Name:	LogStatus
; Description:		Gestion de l'affichage et de l'historique du log
; Parameters:		chaine
;===============================================================================
LogStatus(LogMessage, _fileicon="", _filenumber=1)
{
	global s_LogMsg, LogDisplay
	s_LogMsg .= LogMessage . "|"
	If _fileicon
		SB_SetIcon(_fileicon, _filenumber)
	SB_SetText(LogMessage)
	GuiControl, 3: , LogDisplay, %LogMessage%||
Return
}

; Conversion HTML vers texte
;===============================================================================
; Function Name:	UnHTM
; Description:		Enlève toutes les balises HTML d'une chaine
; 					Convertit vers un texte ordinaire
; Parameters:		chaine
;===============================================================================
UnHTM(HTM) { ; Remove HTML formatting / Convert to ordinary text     by SKAN 19-Nov-2009
 Static HT     ; Forum Topic: www.autohotkey.com/forum/topic51342.html
 IfEqual,HT,,   SetEnv,HT, % "&aacuteá&acircâ&acute´&aeligæ&agraveà&amp&aringå&atildeã&au"
 . "mlä&bdquo„&brvbar¦&bull•&ccedilç&cedil¸&cent¢&circˆ&copy©&curren¤&dagger†&dagger‡&deg"
 . "°&divide÷&eacuteé&ecircê&egraveè&ethð&eumlë&euro€&fnofƒ&frac12½&frac14¼&frac34¾&gt>&h"
 . "ellip…&iacuteí&icircî&iexcl¡&igraveì&iquest¿&iumlï&laquo«&ldquo“&lsaquo‹&lsquo‘&lt<&m"
 . "acr¯&mdash—&microµ&middot·&nbsp &ndash–&not¬&ntildeñ&oacuteó&ocircô&oeligœ&ograveò&or"
 . "dfª&ordmº&oslashø&otildeõ&oumlö&para¶&permil‰&plusmn±&pound£&quot""&raquo»&rdquo”&reg"
 . "®&rsaquo›&rsquo’&sbquo‚&scaronš&sect§&shy­&sup1¹&sup2²&sup3³&szligß&thornþ&tilde˜&tim"
 . "es×&trade™&uacuteú&ucircû&ugraveù&uml¨&uumlü&yacuteý&yen¥&yumlÿ"
 HTM := RegExReplace(HTM, "<br[^>]*>", "`n")
 StringReplace, HTM, HTM, </p><p>, `n, All
 TXT := RegExReplace( HTM,"<[^>]+>" )               ; Remove all tags between  "<" and ">"
 Loop, Parse, TXT, &`;                              ; Create a list of special characters
   L := "&" A_LoopField ";", R .= (!(A_Index&1)) ? ( (!InStr(R,L,1)) ? L:"" ) : ""
 StringTrimRight, R, R, 1
 Loop, Parse, R , `;                                ; Parse Special Characters
  If F := InStr( HT, A_LoopField )                  ; Lookup HT Data
    StringReplace, TXT,TXT, %A_LoopField%`;, % SubStr( HT,F+StrLen(A_LoopField), 1 ), All
  Else If ( SubStr( A_LoopField,2,1)="#" )
    StringReplace, TXT, TXT, %A_LoopField%`;, % Chr(SubStr(A_LoopField,3)), All
Return RegExReplace( TXT, "(^\s*|\s*$)")            ; Remove leading/trailing white spaces
}

;===============================================================================
; Function Name:	TxtToHTMLite
; Description:		Convertit un texte simple en HTML basique
; Parameters:		chaine
;===============================================================================
TxtToHTMLite(_String)
{
	StringReplace, _String, _String, `r`n, `n, All
	StringReplace, _String, _String, `n`n, </p><p>, All
	StringReplace, _String, _String, `n, <br />, All
	StringReplace, _String, _String, %A_Tab%, , All
	_String := "<p>" . _String . "</p>"
Return %_String%
}

;===============================================================================
; Function Name:	SumFolderSize
; Description:		Calcule la taille d'un dossier de manière récursive
; Parameters:		chemin
;===============================================================================
SumFolderSize(Folder)
{
	; On calcule la taille du dossier décompressé
	SetBatchLines, -1
	v = 0
	Loop, %Folder%\*.*, , 1
		v += %A_LoopFileSize%
Return v
}

;===============================================================================
; Function Name:	WM_MOUSEMOVE
; Description:		Affiche et masque les ToolTip en fonction du contrôle
; Parameters:		Aucun
;===============================================================================
WM_MOUSEMOVE()
{
	static CurrControl, PrevControl, _TT  ; _TT is kept blank for use by the ToolTip command below.
	CurrControl := A_GuiControl

	If (CurrControl <> PrevControl and not InStr(CurrControl, " "))
	{
		ToolTip  ; Turn off any previous tooltip.
		SetTimer, DisplayToolTip, 1000
		PrevControl := CurrControl
	}
	return

	DisplayToolTip:
	If (CurrControl = "b_EditScript")
		Gosub UpdateToolTipMsg
	SetTimer, DisplayToolTip, Off
	; on évite les controles dont le nom contient des caractères interdits
	If !RegExMatch(CurrControl, "[^a-zA-Z0-9_]")
		{
		ToolTip % %CurrControl%_TT  ; The leading percent sign tell it to use an expression.
		SetTimer, RemoveToolTip, 3000
		}
	return

	RemoveToolTip:
	SetTimer, RemoveToolTip, Off
	ToolTip
	return

}

;===============================================================================
; Function Name:	CreateSection
; Description:		Formate une chaine de caractères sous forme de section de 
; 					fichier ini avec la liste des clés et leurs valeurs
; Parameters:		nom de la section, liste d'items
;===============================================================================
CreateSection(_Section, _List)
{
	s_SectionString := "[" . _Section . "]"
	Sort, _List, U CL
	i_Count := 0
	Loop, Parse, _List, `n
		{
		; on évite de traiter en cas de ligne vide (pas sûr que cela soit utile avec le Sort précédent)
		If A_LoopField
			{
			i_Count += 1
			s_SectionString .= "`nname" . i_Count . "=" . A_LoopField
			}
		}
Return s_SectionString
}

;===============================================================================
; Function Name:	VerifyKeys
; Description:		Vérifie l'existence de clés dans la section d'un
; 					fichier ini et les crée si besoin
; Parameters:		référence du fichier, nom de la section, liste d'items (séparateur | )
;===============================================================================
VerifyKeys(ByRef _File, _Section, _List)
{
	s_ListKeys := ini_getAllKeyNames(_File, _Section)
	Loop, parse, _List, |
		{
		IfNotInString, s_ListKeys, %A_LoopField%
			ini_insertKey(_File, _Section, A_LoopField . "=")
		}
Return 
}


;===============================================================================
; Function Name:	LogRunWait
; Description:		exécute une commande et renvoie StdOut
; Parameters:		la commande à exécuter
;===============================================================================
LogRunWait(_CmdLine)
{
	; on déclare le chemin de gclip
	p_gclip := A_ScriptDir . "\Tools\UnxUtils\gclip.exe"
	; on efface le presse-papiers
	clipboard =
	; on lance la commande dans une invite invisible et on pipe vers gclip
	RunWait, %comspec% /c "%_CmdLine% | "%p_gclip%"", ,Hide
	If ErrorLevel
		clipboard .= "`r`nExitCode=" . ErrorLevel
	; on retourne le contenu du presse-papiers
Return clipboard
}


;===============================================================================
; Function Name:	NormLink
; Description:		normalise un lien pour PmWiki
; Parameters:		le nom de l'application portable
;===============================================================================
NormLink(_page)
{
	_page := StringReplace(_page, "+", "plus", "All")
Return "www.framakey.org/Portables/" . _page
}



;===============================================================================
; Function Name:	UpImgStatus
; Description:		Met à jour l'état du contrôle pour une image
; Parameters:		nom du contrôle, nom du fichier
;===============================================================================
UpImgStatus(_control, _file)
{
	_value := (FileExist(_file)) ? "Présent" : "Absent"
	GuiControl,, % _control, %_value%
	If (_value = "Présent")
		GuiControl, +cLime, % _control
	Else
		GuiControl, +cRed, % _control
	GuiControl, MoveDraw, % _control
Return 
}